/************************************************************************
 *
 * Copyright (C) 1986-1998 Micro Focus Ltd.
 *
 * All rights reserved.  Unpublished, all rights reserved under
 * copyright law and international treaty.  Use of a copyright notice
 * is precautionary only and does not imply publication or disclosure.
 * This software contains confidential information and trade secrets of
 * Micro Focus Ltd. and no part of this program or publication may be
 * reproduced, transmitted, transcribed, stored in a retrieval system,
 * or translated into any language or computer language, in any form
 * or by any means, electronic, mechanical, magnetic, optical, chemical,
 * manual or otherwise, without the prior written permission of:
 *
 * Micro Focus Ltd,
 * The Lawn, Old Bath Road, Newbury, Berkshire, RG14 1QN, UK.
 * (01635) 32646
 *
 * RESTRICTED RIGHTS LEGEND
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights
 * in Technical Data and Computer Software clause at DFARS 52.227-7013
 * or subparagraphs (c)(1) and (2) of Commercial Computer Software -
 * Restricted Rights at 48 CFR 52.227-19, as applicable.  Contact
 * Micro Focus Incorporated, 2465 East Bayshore Road, Palo Alto,
 * California 94303.
 *
 *
 ************************************************************************
 *
 * Start of source module cmain1.c
 *
 ************************************************************************/

#include <stdio.h>
#include "cobmain.h"
#include "cobcall.h"

#include "cmain1.h"

int main()
{
    struct param1 param;
    struct param2 paramb;
    char text[80];
    void * ptr;


    /**
    *** First off, lets just let everyone know we're starting
    *** in the correct place.
    **/
    printf("In C main1\n");

    /**
    *** Now we need to initialise the Cobol system before we
    *** start making any calls to Cobol.
    **/
    cobinit();


    /**
    *** ************************************************************
    *** The first call to a function is to be made using the
    *** cobcall facility. In this we need to pass a char**
    *** which points to our list of parameters. In this case
    *** we're only passing one parameter, which is the address
    *** of a structure.
    *** ************************************************************
    **/

    /**
    *** ...make sure the structure is empty.
    **/
    param.a     = 0;
    param.b     = 0;
    param.c     = 0;
    param.d     = '\0';

    /**
    *** Setup a pointer to the address of our structure.
    *** We can pass the address of the pointer to cobcall
    *** as a (char**).
    **/
    ptr = (void*)&param;

    /**
    *** ...call that function with 1 parameter
    **/
    cobcall("cobsub1", 1, (char**)&ptr);

    /**
    *** ...and now cancel it, as we don't need it anymore...
    **/
    cobcancel("cobsub1");

    /**
    *** Now we just check to make sure that the COBOL function
    *** has changed our parameters as expected.
    **/
    if((param.a == 10.5) && (param.b == 10) &&
					(param.c == 10) && (param.d == 'a'))
	printf("PASS\n");
    else
	printf("FAIL\n");


    /**
    *** ************************************************************
    *** ...now we'll try again without using cobcall. Instead we'll
    *** attempt a straight C call and see if the cob -d.... option
    *** has worked correctly.
    *** ************************************************************
    **/

    /**
    *** ...make sure the structure is empty.
    **/
    param.a     = 0;
    param.b     = 0;
    param.c     = 0;
    param.d     = '\0';

    /**
    *** ...call that function with 1 parameter
    **/
    cobsub2(&param);

    /**
    *** Now we just check to make sure that the COBOL function
    *** has changed our parameters as expected.
    **/
    if((param.a == 10.5) && (param.b == 10) &&
					(param.c == 10) && (param.d == 'a'))
	printf("PASS\n");
    else
	printf("FAIL\n");

    /**
    *** ************************************************************
    *** This last call to a COBOL function is similar to the
    *** previous call, except paramb.p_pid is a pid_t type. This
    *** also shows the advantages of h2cpy.
    *** ************************************************************
    **/

    /**
    *** We'll setup some useless info for COBOL to check this time.
    **/
    strcpy(text, "Here is some text");
    paramb.p_pid = getpid();
    paramb.p_status = 'a';
    paramb.p_string = text;
    paramb.p_price = 10.34;

    /**
    *** ...and then call the function.
    **/
    cobsub3(&paramb);

    /**
    *** ...and just check if all went well.
    **/
    if(paramb.p_status == 'b')
	printf("PASS\n");
    else
	printf("FAIL\n");

    /**
    *** Now we just need to deinitialise the Cobol system.
    **/
    cobtidy();

    printf("Bye\n");
    return 0;
}

