      $set mfoo ooctrl(+n)
      *------------------------------------------------------------
      * HighRateAccount  is a sub-class derived from the class
      * SavingsAccount.
      *
      * Copyright (c) Micro Focus 1991 - 1999.
      *------------------------------------------------------------
       class-id. HighRateAccount
                       data is protected
                       inherits from SavingsAccount with data.

        object section.
        class-control.
           highRateAccount is class "haccount"
           SavingsAccount is class "saccount"
           ExceptionManager is class "exptnmgr"
           .
       working-storage section.
           copy "accinfo.cpy".
       procedure division.
      *----H000. Initialize the class.
           invoke self "initializeClass"
       class-object.
       object-storage section.

      *------------------------------------------------------------
      * Method OpenAccount overides the method OpenAccount of its base
      * class Account. This is because opening the High Interest
      * Account requires a minimum deposit of $1000, which behaves
      * differently from other accounts.
      *------------------------------------------------------------

       method-id. "openAccount".
       local-storage section.
           01 lsErrorNumber               pic x(4) comp-5.

       linkage section.
           01 lsInitialValue               pic 9(10)v99.
           01 lsName pic x(10).
           01 lsAccount                    usage object reference.

       procedure division using lsName lsInitialValue
                      returning lsAccount.
      *----H010. Check that the account is being opened with enough
      *          money.
           if lsInitialValue < 1000  then
      *--------H011. Raise an exception if there are insufficient funds
      *              to open the account. If there is no exception
      *              handler in place to trap this error, the default
      *              exception handler stops the application.
      *----H002. Get error offset and raise exception.
               invoke ExceptionManager "queryMessageFile"
                                       using "bank" "account"
                                   returning lsErrorNumber
               add insufficientFundsforAccount to lsErrorNumber
               invoke self "raiseException" using lsErrorNumber
                                        returning lsAccount
           else
      *--------H012. Open the account using the inherited method.
               invoke super "openAccount" using lsName lsInitialValue
                                      returning lsAccount
           end-if
           exit method.
       end method "openAccount".

      *------------------------------------------------------------
      * Set the first account number for high rate accounts.
      *------------------------------------------------------------
       Method-id. "getFirstAccountNumber".

       linkage section.
           01 lnkNumber               pic 9(8).
           01 lsErrorOffset           pic x(4) comp-5.

       procedure division returning lnkNumber.
      *----H001. Set up account numbering for high rate accounts.
           move 40000000 to lnkNumber
           exit method.
       end method "getFirstAccountNumber".
      *------------------------------------------------------------
      * Return the interest rate of this type of account
      *------------------------------------------------------------
       Method-id. "getRate".

       linkage section.
           01 lnkRate      pic 99v99 comp.
       procedure division returning lnkRate.
           move 10 to lnkRate
           exit method.
       end method "getRate".
       end class-object.
       Object.
       object-storage section.

      *------------------------------------------------------------
      * Print a statement to the display
      *------------------------------------------------------------
       method-id. "printStatement".
       local-storage section.

           01  displayBalance       pic -(9)9.99.

       procedure division.
      *----C020. Print a statement.
           display "High Rate Account"
           invoke self "updateBalance"
           display "Name:          " aName
           move balance to displayBalance
           display "Balance:       " displayBalance
           move interest to displayBalance
           display "Interest paid: " displayBalance
           exit method.
       end method "printStatement".
      *-----------------------------------------------------------------
      * Return the type of account.
      *-----------------------------------------------------------------
       method-id. "getType".

       linkage section.
           01  lsType          pic x(accTypeLength).

       procedure division returning lsType.
      *----H030. Return the account type.
           move spaces to lsType
           move highRateType to lsType
           exit method.
       end method "getType".
       end object.
       end class HighRateAccount.
