      $set sql(dbman=ODBC, detectdate)
      ****************************************************************
      *  Copyright Micro Focus Limited 1996.    All Rights Reserved.
      *  This demonstration program is provided for use by users of
      *  Micro Focus products and may be used, modified and
      *  distributed as part of your application provided that you
      *  properly acknowledge the copyright of Micro Focus in this
      *  material.
      *
      *---------------------------------------------------------------
       working-storage section.

      *  Include the SQL Communications Area. This includes the
      *  definitions of SQLCODE, etc
       exec sql include sqlca end-exec

       01 Disp-Salary          pic z(8)9.99.

       exec sql begin declare section end-exec	
       
       01 First-Name           pic x(8).
       01 Last-Name            pic x(10).
       01 Emp-Id               pic x(6).
       01 Hire-Date            pic x(26).
       01 Salary               pic 9(9)v9(2).
       01 Dept                 pic x(4).

       01 svr                  pic x(32).
       01 usr                  pic x(32).
       01 pass                 pic x(32).
       01 usr-pass             pic x(64).

       exec sql end declare section end-exec	

       procedure division.
           display "Enter data source name "
                   with no advancing
           accept svr
           display "Enter username "
                    with no advancing
           accept usr
           display "Enter password "
                    with no advancing
           accept pass
           string
               usr delimited space
               "." delimited size
               pass delimited space
           into usr-pass
      *    Specify the routine to go to if an error 
      *    occurs in a sql statement
	   exec sql
	       whenever sqlerror go to Sql-Error
           end-exec	
      *    Connect to the specified database
           exec sql
	       connect to :svr user :usr-pass with no prompt	
	   end-exec

      *    Example of the insert statement using literals	
           display 'Insert row using literals'
           exec sql
               insert into EMP 
                           (FIRST_NAME,
                            LAST_NAME,
                            EMP_ID,
  			    HIRE_DATE,
                            SALARY,
                            DEPT)
                     values ('John',
                             'Andrews',
                             'E17243',
                             {d'1989-10-06'},
                             35000.00,
                             'D101')
           end-exec

           display 'Insert using host variables'
           move "Joe" to First-Name
           move "Long" to Last-Name
           move "{d'1988-01-21'}" to Hire-Date
           move "E54621" to Emp-Id
           move 45000.00 to Salary
           move "D101" to Dept
           exec sql
               insert into EMP
                           (FIRST_NAME,
                            LAST_NAME,
                            EMP_ID,
                            HIRE_DATE,
                            SALARY,
                            DEPT)
                     values (:First-Name,
                             :Last-Name,
                             :Emp-Id,
                             :Hire-Date,
                             :Salary,
                             :Dept)
           end-exec

           display 'Commit the inserts'
      *    Commit the inserts to ensure that the changes
      *    are permanent
           exec sql
               commit transaction
           end-exec

           display 'Cursor select'
      *    Declare and open a cursor to retrieve information
      *    on employees who are in department '0101'
           exec sql
               declare EmpCursor cursor for
                   select FIRST_NAME,
                          LAST_NAME,
                          HIRE_DATE,
                          SALARY
                     from EMP
                   where DEPT = 'D101'
           end-exec

           exec sql
               open EmpCursor
           end-exec
           display ' '
           display 'First Name  Last Name   Hire Date         Salary'
           display '----------  ---------   ---------   ------------'

      *    Retrieve the employee information
           perform until sqlcode = 100
               exec sql
                   fetch EmpCursor into :First-Name,
                                        :Last-Name,
                                        :Hire-Date,
                                        :Salary
               end-exec
               if sqlcode not = 100
                   move Salary to Disp-Salary
                   display First-Name '    '
                           Last-Name  '  '
                           Hire-Date(1:10) '  '
                           Disp-Salary
               end-if
           end-perform
      *    Close the cursor
           exec sql
               close EmpCursor
           end-exec
           display '  '
           display 'Update row'

      *    Update one of the inserted rows
           exec sql
               update EMP
                   set SALARY = 50000.00
                   where LAST_NAME = 'Long' and
                         FIRST_NAME = 'Joe'
           end-exec

      *    Verify that the data was updated successfully
           display 'Verify data before rollback'
           exec sql
               select SALARY into :Salary FROM EMP
                   where LAST_NAME = 'Long' and
                         FIRST_NAME = 'Joe'
           end-exec
           if Salary = 50000
               display "Update was successful"
           else
               display "Update was not successful"
           end-if

           display 'Rollback the update'
      *    Rollback the update
           exec sql
               rollback
           end-exec

           display 'Verify data after rollback'
           exec sql
               select SALARY into :Salary FROM EMP
                   where LAST_NAME = 'Long' and
                         FIRST_NAME = 'Joe'
           end-exec
           if Salary = 45000
               display "Rollback was successful"
           else
               display "Rollback was not successful"
           end-if

           display 'Delete inserted rows'
      *    Now, delete the rows we inserted earlier. One
      *    statement uses literals and the other uses
      *    host variables
           exec sql
               delete from EMP
                   where LAST_NAME = 'Andrews' and
                         FIRST_NAME = 'John'
           end-exec
           move "Long" to Last-Name
           move "Joe" to First-Name
           exec sql
               delete from EMP
                   where LAST_NAME = :Last-Name and
                         FIRST_NAME = :First-Name
           end-exec

      *    Commit the changes
           exec sql
               commit transaction
           end-exec

           display 'Disconnect'
           exec sql
               disconnect current
           end-exec
           stop run.

       Sql-Error section.
           display "Error occured in SQL Statement".
           display "Sqlcode = " sqlcode
           display "Sqlerrmc = " sqlerrmc
           stop run.
              			
