/****************************************************************************
 *
 *	Copyright (C) 1984-1998 Micro Focus Ltd.
 *
 *	All rights reserved.  Unpublished, all rights reserved under
 *	copyright law and international treaty.  Use of a copyright notice
 *	is precautionary only and does not imply publication or disclosure.
 *	This software contains confidential information and trade secrets of
 *	Micro Focus Ltd. and no part of this program or publication may be
 *	reproduced, transmitted, transcribed, stored in a retrieval system,
 *	or translated into any language or computer language, in any form
 *	or by any means, electronic, mechanical, magnetic, optical, chemical,
 *	manual or otherwise, without the prior written permission of:
 *
 *	Micro Focus Ltd,
 *	The Lawn, Old Bath Road, Newbury, Berkshire, RG14 1QN, UK.
 *	(01635) 32646
 *
 *	RESTRICTED RIGHTS LEGEND
 *	Use, duplication, or disclosure by the Government is subject to
 *	restrictions as set forth in subparagraph (c)(1)(ii) of the Rights
 *	in Technical Data and Computer Software clause at DFARS 52.227-7013
 *	or subparagraphs (c)(1) and (2) of Commercial Computer Software -
 *	Restricted Rights at 48 CFR 52.227-19, as applicable.  Contact
 *	Micro Focus Incorporated, 701 East Middlefield Road, Mountain View,
 *	California 94043, USA.
 *
 *	Genesis Run Time System (M3/Hawaii Freeze)
 *
 *	@(#)mf_types.h	3.7
 *
 *	Last Modified on 98/10/15 at 15:52:37
 *
 ****************************************************************************/

/** Start of source module mf_types.h **/

#ifndef	MF_TYPES
#define	MF_TYPES

#include <stddef.h>
#include <sys/types.h>

typedef unsigned char	mf_uns8;
typedef unsigned short	mf_uns16;
typedef short		mf_s16;
typedef unsigned int	mf_uns32;
typedef int 		mf_s32;
typedef double		mf_flt;	 /* Floating point data type. Max precision */
typedef void		*mf_sigtype;
typedef void		*mf_eptype;

typedef	int		mf_boolean;

typedef int		(*PFI)(); /* pointer to function returning int	*/
typedef long		(*PFL)(); /* pointer to function returning long */
typedef void		(*PFV)(); /* pointer to void function		*/
typedef void	       *(*PFVP)();/* pointer to void *function		*/

/* Common function types with single or no params */

typedef int	  (*PFI_VOID)(void);		/* int func: no params */
typedef int	  (*PFI_INT)(int);		/* int func: int param */
typedef int	  (*PFI_UNS8)(mf_uns8);		/* int func: uns8 param */
typedef int	  (*PFI_UNS16)(mf_uns16);	/* int func: uns16 param */
typedef int	  (*PFI_UNS32)(mf_uns32);	/* int func: uns32 param */
typedef int	  (*PFI_PUNS8)(mf_uns8 *);	/* int func: uns8* param */
typedef int	  (*PFI_PUNS16)(mf_uns16 *);	/* int func: uns16* param */
typedef int	  (*PFI_PVOID)(void *);		/* int func: void* param */
typedef int	  (*PFI_PVOID_PVOID)(void *, void *);
						/* int func: 2 x void*    */

typedef mf_uns8	 *(*PFPUNS8_VOID)(void);	/* uns8* func: no params */
typedef mf_uns8	 *(*PFPUNS8_PINT)(int *);	/* uns8* func: int* param */

typedef void	  (*PFV_VOID)(void);		/* void func: no params */
typedef void	  (*PFV_INT)(int);		/* void func: int param */
typedef void	  (*PFV_PUNS8)(mf_uns8 *);	/* void func: uns8* param */
typedef void	  (*PFV_PVOID)(void *);		/* void func: void* param */
typedef void	  (*PFV_PVOID_PVOID)(void *, void *);
						/* void func: 2 x void* */

typedef void	 *(*PFPV_VOID)(void);		/* void* func: no params */
typedef void	 *(*PFPV_PVOID)(void *);	/* void* func: void* param */

typedef PFI	(*PFPFI_PUNS8_INT)(mf_uns8 *, int);
						/* PFI func: uns8*, int */

#ifdef	NOPID_T
typedef	mf_s32		pid_t;
#endif /* NOPID_T */

#ifdef NOCADDR_T
typedef	mf_uns8			*caddr_t;
#endif /* NOCADDR_T */

#ifdef NOSIZE_T
typedef mf_uns32	size_t;
#endif /* NOSIZE_T */

#ifdef NOPTRDIFF_T
typedef mf_uns32	ptrdiff_t;
#endif /* NOSIZE_T */

#ifndef OSTIME_DEFINED
typedef mf_uns32	mf_time_t;
#endif /* OSTIME_DEFINED */

/* 
 * Definitions of 64-bit values used primarily in file handling. 
 * Use native types where available, otherwise default to a structure 
 * consisting of two 32-bit values. To accomodate both without penalising
 * systems with native types, macros are included for declaring and accessing
 * the values. The latter is required to allow access to items declared in
 * structures (eg filesystem_statbuff).
 */

#ifdef MF_INT64

typedef MF_INT64		mf_s64, mf_s64_t;
typedef unsigned MF_INT64	mf_uns64, mf_uns64_t;

#define INT64_DECL(x)		mf_s64 x;
#define UINT64_DECL(x)		mf_uns64 x;
#define INT64_VAL(x)		x

#else /* MF_INT64 */

typedef struct 
{
    mf_uns32			mf_lo32;
    mf_s32			mf_hi32;
} mf_s64_t, *mf_s64;

typedef struct
{
    mf_uns32			mf_lo32;
    mf_uns32			mf_hi32;
} mf_uns64_t, *mf_uns64;

#define INT64_DECL(x)		mf_s64_t x ## _ ; mf_s64 x = &x ## _ ;
#define UINT64_DECL(x)		mf_uns64_t x ## _ ; mf_uns64 x = &x ## _;
#define INT64_VAL(x)		&(x)

#endif /* MF_INT64 */

/*
** Types to handle easy transition to 64 bit pointers and sizes
**/

typedef size_t   	mf_size;

typedef ptrdiff_t	mf_ptrdiff;
typedef ptrdiff_t	mf_ptrnum;

#if defined(OS_P64BIT) && !defined(COBOL_64BIT) 
typedef mf_uns32        mf_cblptr;  /* Assume 32 bit COBOL pointers */
#else 
typedef mf_uns8        *mf_cblptr;
#endif

#endif	/* MF_TYPES */

/** End of source module mf_types.h **/
